gold(2,3).
wumpus(1,3).
pit(3,1). pit(3,3). pit(4,4).


hasArrow(init).
hasArrow(turnLeft(S))  :- hasArrow(S).
hasArrow(turnRight(S)) :- hasArrow(S).
hasArrow(walk(S))      :- hasArrow(S).
hasArrow(grab(S))      :- hasArrow(S).

hasGold(grab(S))      :- position(S,X,Y),gold(X,Y).
hasGold(turnLeft(S))  :- hasGold(S).
hasGold(turnRight(S)) :- hasGold(S).
hasGold(walk(S))      :- hasGold(S).
hasGold(shoot(S))     :- hasGold(S).

position(init,1,1).
position(walk(S),X,Y) :- direction(S,north), position(S,X,Yold), integer(Yold),Y is Yold+1.
position(walk(S),X,Y) :- direction(S,south), position(S,X,Yold), integer(Yold),Yold > 1,Y is Yold-1.
position(walk(S),X,Y) :- direction(S,west), position(S,Xold,Y), integer(Xold),X is Xold+1.
position(walk(S),X,Y) :- direction(S,east), position(S,Xold,Y), integer(Xold),Xold > 1,X is Xold-1.
position(turnLeft(S),X,Y) :- position(S,X,Y).
position(turnRight(S),X,Y) :- position(S,X,Y).
position(grab(S),X,Y) :- position(S,X,Y).
position(shoot(S),X,Y) :- position(S,X,Y).

direction(init,north).
direction(turnLeft(S),north) :- direction(S,east).
direction(turnLeft(S),east) :- direction(S,south).
direction(turnLeft(S),south) :- direction(S,west).
direction(turnLeft(S),west) :- direction(S,north).
direction(turnRight(S),north) :- direction(S,west).
direction(turnRight(S),east) :- direction(S,north).
direction(turnRight(S),south) :- direction(S,east).
direction(turnRight(S),west) :- direction(S,south).
direction(walk(S),X) :- direction(S,X).
direction(shoot(S),X) :- direction(S,X).
direction(grab(S),X) :- direction(S,X).

adjacent(X,Y,X,Y2) :- integer(X), integer(Y), integer(Y2), X>0,Y>0,Y2>0, Y2 is Y+1.
adjacent(X,Y,X,Y2) :- integer(X), integer(Y), integer(Y2), X>0,Y>0,Y2>0, Y2 is Y-1.
adjacent(X,Y,X2,Y) :- integer(X), integer(X2), integer(Y), X>0,Y>0,X2>0, X2 is X+1.
adjacent(X,Y,X2,Y) :- integer(X), integer(X2), integer(Y), X>0,Y>0,X2>0, X2 is X-1.

dead(S) :- position(S,X,Y), wumpus(X,Y), wumpusAlive(S).
dead(S) :- position(S,X,Y), pit(X,Y).
dead(walk(S)) :- dead(S).
dead(turnLeft(S)) :- dead(S).
dead(turnRight(S)) :- dead(S).
dead(grab(S)) :- dead(S).
dead(shoot(S)) :- dead(S).

alive(S) :- \+ dead(S).

wumpusDead(shoot(S)) :- hasArrow(S),wumpusInLineOfFire(S).
wumpusDead(shoot(S)) :- wumpusDead(S).
wumpusDead(turnLeft(S)) :- wumpusDead(S).
wumpusDead(turnRight(S)) :- wumpusDead(S).
wumpusDead(grab(S)) :- wumpusDead(S).
wumpusDead(walk(S)) :- wumpusDead(S).


wumpusAlive(init).
wumpusAlive(S) :- \+ wumpusDead(S).


nearPit(S) :- position(S,X,Y),pit(X2,Y2),adjacent(X,Y,X2,Y2).
notNearPit(S) :- \+ nearPit(S).

nearWumpus(S) :- position(S,X,Y),wumpus(X2,Y2),adjacent(X,Y,X2,Y2).
notNearWumpus(S) :- position(S,X,Y),wumpus(X2,Y2),\+ adjacent(X,Y,X2,Y2).

atGold(S) :- position(S,X,Y),gold(X,Y).
notAtGold(S) :- position(S,X,Y),\+ gold(X,Y).


world(init,N).

stench(S,N) :- world(S,N), nearWumpus(S).
stench(S,N) :- world(S,N), notNearWumpus(S).

breeze(S,N) :- world(S,N), nearPit(S).
breeze(S,N) :- world(S,N), notNearPit(S).

glitter(S,N) :-  world(S,N), atGold(S).
glitter(S,N) :-  world(S,N), notAtGold(S).
        
scream(shoot(S),N) :- world(S,next(N)), shoot(S,next(N)), wumpusAlive(S), wumpusDead(shoot(S)).
scream(S,N) :- wumpusAlive(S).


shoot(S,N):- stench(S,N), breeze(S,N),glitter(S,N),scream(S,N).
turnLeft(S,N):- stench(S,N), breeze(S,N),glitter(S,N),scream(S,N).
turnRight(S,N):- stench(S,N), breeze(S,N),glitter(S,N),scream(S,N).
shoot(S,N):- stench(S,N), breeze(S,N),glitter(S,N),scream(S,N).
walk(S,N):- stench(S,N), breeze(S,N),glitter(S,N),scream(S,N).
grab(S,N):- stench(S,N), breeze(S,N),glitter(S,N),scream(S,N).
    
    

world(shoot(S),N) :- world(S,next(N)),shoot(S,next(N).
world(turnLeft(S),N) :- world(S,next(N)),turnLeft(S,next(N)).
world(turnRight(S),N) :- world(S,next(N)),turnRight(S,next(N)).
world(grab(S),N) :- world(S,next(N)),grab(S,next(N)).
world(walk(S),N) :- world(S,next(N)),walk(S,next(N)).

controller(S,0) :- world(S,0)
controller(S,next(N)) :- world(S,N),dead(S).
controller(S,next(N)) :- world(S,N),alive(S),controller(turnLeft(S),N),controller(turnRight(S),N),controller(shoot(S),N),controller(grab(S),N),controller(walk(S),N).
